HonorTracker = select(2, ...)

local L = HonorTracker.L

-- Replace DKs text with estimated honor
hooksecurefunc("HonorFrame_Update", function(updateAll) HonorTracker:UpdatePVPFrame() end)
HonorFrame:SetScript("OnShow", function() HonorTracker:UpdatePVPFrame() end)

-- Add our tooltips
GameTooltip:HookScript("OnTooltipSetUnit", function(self, unit)
	if (not HonorTrackerDB or not HonorTrackerDB.config.tooltips) then return end

	local name, unit = self:GetUnit()
	if (not unit or not name) then return end

	if (UnitIsPlayer(unit) and UnitFactionGroup(unit) ~= UnitFactionGroup("player")) then
		local trackingData = HonorTrackerDB.tracking.players[name]
		if (not trackingData) then return end

		local killColor = "|cFFFF0000"
		if (trackingData.kills == 1) then
			killColor = "|cFF00FF00"
		elseif (trackingData.kills == 2) then
			killColor = "|cFFFFF468"
		elseif (trackingData.kills == 3) then
			killColor = "|cFFFF7C0A"
		elseif (trackingData.kills >= 4) then
			killColor = "|cFFFF0000"
		end

		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(string.format(L["|cFFFFFFFFKilled:|r %s%d|r |cFFFFFFFF(honor: %d)|r"], killColor, trackingData.kills, trackingData.honor))
		GameTooltip:Show()
	end
end)

-- Make honor gain messages useful
ChatFrame_AddMessageEventFilter("CHAT_MSG_COMBAT_HONOR_GAIN", function(chatFrame, event, text, ...)
	local name, _other = string.split(" ", text, 2)
	local honor = tonumber(string.match(text, L["Estimated Honor Points: (%d+)"]))

	local actualHonor = HonorTracker:CalculateModifiedHonor(name, honor)
	
	text = string.gsub(
		text,
		L["Estimated Honor Points: (%d+)"],
		string.format(
			L["Estimated: %d, Actual: %d, Kills: %d"],
			honor,
			actualHonor and math.ceil(actualHonor) or "??",
			(HonorTrackerDB.tracking.players[name] and HonorTrackerDB.tracking.players[name].kills or "??")
		)
	)

	return false, text, ...
end)

function HonorTracker:UpdatePVPFrame()
	if (not HonorFrameCurrentDKValue:IsVisible()) then return end
	
	local stats = HonorTracker:CalculateStats()
	
	local text = L["Estimates from HonorTracker"] .. "\n\n"
	text = text .. string.format(L["|c%sRecorded Kills:|r %d"] .. "\n", HIGHLIGHT_FONT_COLOR:GenerateHexColor(), stats.recordedKills)
	text = text .. string.format(L["|c%sAvg Honor Per Kill:|r %.2f"] .. "\n", HIGHLIGHT_FONT_COLOR:GenerateHexColor(), stats.avgHonorPerKill)
	text = text .. "\n"
	text = text .. string.format(L["|c%sPlayers Killed:|r %d"] .. "\n", HIGHLIGHT_FONT_COLOR:GenerateHexColor(), stats.uniquePlayers)
	text = text .. string.format(L["|c%sAvg Kills Per Player:|r %.2f"] .. "\n", HIGHLIGHT_FONT_COLOR:GenerateHexColor(), stats.avgKillsPerPlayer)
	text = text .. string.format(L["|c%sAvg Honor Per Player:|r %.2f"], HIGHLIGHT_FONT_COLOR:GenerateHexColor(), stats.avgHonorPerPlayer)

	HonorFrameCurrentDK.title = L["Estimated Honor"]
	HonorFrameCurrentDK.tooltip = text

	local text = ""
	for id, stats in pairs(HonorTrackerDB.tracking.thisWeekStats) do
		if(text ~= "") then text = text .. "\n" end
		text = text .. string.format(L["|cFFFFFFFF%s:|r %d honor, %d kills"], date("%A", stats.time), stats.actualHonor, stats.recordedKills)
	end

	HonorFrameThisWeekContribution.text = L["This Weeks Daily Stats"]
	HonorFrameThisWeekContribution.tooltip = text

	HonorFrameCurrentDKText:SetText(L["Estimated Honor"])
	HonorFrameCurrentDKValue:SetText(HonorTrackerDB.tracking.today)
	HonorFrameCurrentDKValue:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
end

function HonorTracker:CalculateModifiedHonor(name, honor)
	if (not HonorTrackerDB.tracking.players[name]) then
		return honor
	elseif (HonorTrackerDB.tracking.players[name].kills == 1) then
		return honor
	elseif (HonorTrackerDB.tracking.players[name].kills == 2) then
		return math.floor((honor * 0.75) + 0.5)
	elseif (HonorTrackerDB.tracking.players[name].kills == 3) then
		return math.floor((honor * 0.50) + 0.5)
	elseif (HonorTrackerDB.tracking.players[name].kills == 4) then
		return math.floor((honor * 0.25) + 0.5)
	else
		return 0
	end
end

function HonorTracker:CHAT_MSG_COMBAT_HONOR_GAIN(text, playerName, languageName, channelName, playerName2, specialFlags, zoneChannelID, chanenlIndex, channelBaseName, unused, lineID, guid, bnSenderID, isMobile, isSubtitel, hideSenderInLetterbox, suppressRaidIcons)
	local name, _other = string.split(" ", text, 2)

	HonorTrackerDB.tracking.players[name] = HonorTrackerDB.tracking.players[name] or {kills = 0, honor = 0}
	HonorTrackerDB.tracking.players[name].kills = HonorTrackerDB.tracking.players[name].kills + 1

	local honor = tonumber(string.match(text, L["Estimated Honor Points: (%d+)"]))
	local actualHonor = HonorTracker:CalculateModifiedHonor(name, honor)

	HonorTrackerDB.tracking.today = (HonorTrackerDB.tracking.today or 0) + actualHonor
	HonorTrackerDB.tracking.players[name].honor = HonorTrackerDB.tracking.players[name].honor + actualHonor
	self:UpdatePVPFrame()
end

function HonorTracker:CheckDB()
	HonorTrackerDB = HonorTrackerDB or {}
	HonorTrackerDB.config = HonorTrackerDB.config or {tooltips = true} 

	HonorTrackerDB = HonorTrackerDB or {}
	HonorTrackerDB.tracking = HonorTrackerDB.tracking or {}
	HonorTrackerDB.tracking.players = HonorTrackerDB.tracking.players or {}
	HonorTrackerDB.tracking.thisWeekStats = HonorTrackerDB.tracking.thisWeekStats or {}
end

function HonorTracker:DailyReset(actualHonor)
	local offBy
	if( actualHonor > HonorTrackerDB.tracking.today ) then
		offBy = HonorTrackerDB.tracking.today / actualHonor
	else
		offBy = actualHonor / HonorTrackerDB.tracking.today
	end

	HonorTracker:Print(
		string.format(
			L["Estimated Honor: %d, Actual: %d, Off By: %.2f%%"],
			HonorTrackerDB.tracking.today,
			actualHonor,
			(1 - offBy) * 100
		)
	)

	local stats = HonorTracker:CalculateStats()
	HonorTracker:Print(string.format(L["Recorded %d kills (avg %.2f honor per kill) across %d players (%.2f honor per player)"], stats.recordedKills, stats.avgHonorPerKill, stats.uniquePlayers, stats.avgHonorPerPlayer))
end

function HonorTracker:DumpWeeklyStats()
	for id, stats in pairs(HonorTrackerDB.tracking.thisWeekStats) do
		DEFAULT_CHAT_FRAME:AddMessage(
			string.format(
				L["|cFFFFF468%s:|r %d honor (%d estimated), %d kills (avg %.2f honor/kill), %d players (%.2f honor/player)"],
				date("%A", stats.time),
				stats.actualHonor,
				stats.estimatedHonor,
				stats.recordedKills,
				stats.avgHonorPerKill,
				stats.uniquePlayers,
				stats.avgHonorPerPlayer
			)
		)
	end
end

function HonorTracker:CheckDayChange()
	self:CheckDB()

	local hk, dk, honor = GetPVPYesterdayStats()
	if (HonorTrackerDB.tracking.yesterday ~= string.join(", ", GetPVPYesterdayStats())) then
		if (HonorTrackerDB.tracking.today) then
			self:DailyReset(honor)

			-- Record our daily stats
			stats.time = GetServerTime()
			stats.actualHonor = honor
			stats.estimatedHonor = HonorTrackerDB.tracking.today
			table.insert(HonorTrackerDB.tracking.thisWeekStats, stats)
		end

		HonorTrackerDB.tracking.yesterday = string.join(", ", GetPVPYesterdayStats())
		HonorTrackerDB.tracking.today = 0
		HonorTrackerDB.tracking.playersYesterday = HonorTrackerDB.tracking.players
		HonorTrackerDB.tracking.players = {}
	end

	-- Reset our daily stats
	local resetTag = string.join(", ", GetPVPLastWeekStats())
	if (HonorTrackerDB.tracking.lastWeek ~= resetTag) then
		self:Print(L["Last Weeks Daily Stats"])
		self:DumpWeeklyStats()

		HonorTrackerDB.tracking.lastWeek = resetTag
		HonorTrackerDB.tracking.thisWeekStats = {}
	end
end

function HonorTracker:Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Honor Tracker|r: " .. msg)
end

function HonorTracker:CalculateStats()
	local stats = {
		recordedKills = 0,
		uniquePlayers = 0,
		avgKillsPerPlayer = 0,
		avgHonorPerPlayer = 0,
		avgHonorPerKill = 0
	}

	for name, data in pairs(HonorTrackerDB.tracking.players) do
		stats.uniquePlayers = stats.uniquePlayers + 1
		stats.recordedKills = stats.recordedKills + data.kills
	end

	if (stats.recordedKills > 0) then
		stats.avgKillsPerPlayer = math.floor((stats.recordedKills / stats.uniquePlayers) * 100) / 100
	end

	if (HonorTrackerDB.tracking.today and HonorTrackerDB.tracking.today > 0) then
		stats.avgHonorPerKill = math.floor((HonorTrackerDB.tracking.today / stats.recordedKills) * 100) / 100
		stats.avgHonorPerPlayer = math.floor((HonorTrackerDB.tracking.today / stats.uniquePlayers) * 100) / 100
	end

	return stats
end

-- Slash commands
SLASH_HONORTRACKER1 = "/honortracker"
SLASH_HONORTRACKER2 = "/ht"
SlashCmdList["HONORTRACKER"] = function(msg)
	msg = string.lower(msg)
	
	if (msg == "today") then
		HonorTracker:Print(string.format(L["Estimate for today: %d"], math.floor(HonorTrackerDB.tracking.today or 0)))
		
		local stats = HonorTracker:CalculateStats()
		HonorTracker:Print(string.format(L["Recorded %d kills (avg %.2f honor per kill) across %d players (avg %.2f kills and %.2f honor per player)"], stats.recordedKills, stats.avgHonorPerKill, stats.uniquePlayers, stats.avgKillsPerPlayer, stats.avgHonorPerPlayer))

	elseif (msg == "week") then
		HonorTracker:Print(L["This Weeks Daily Stats"])
		HonorTracker:DumpWeeklyStats()

	elseif (msg == "tooltips") then
		HonorTrackerDB.config.tooltips = not HonorTrackerDB.config.tooltips

		if (HonorTrackerDB.config.tooltips) then
			HonorTracker:Print(L["Tooltips for kills are now enabled."])
		else
			HonorTracker:Print(L["Tooltips for kills are now disabled."])
		end
	else
		HonorTracker:Print(L["/honortracker (/ht) commands:"])
		DEFAULT_CHAT_FRAME:AddMessage(L["/ht today - Estimate of your honor earned today"])
		DEFAULT_CHAT_FRAME:AddMessage(L["/ht week - Show a history of your honor gains for the current week"])
		DEFAULT_CHAT_FRAME:AddMessage(L["/ht tooltips - Toggling showing tooltip data for kills"])
	end
end

-- Event handler
local eventFrame = CreateFrame("Frame")
eventFrame:SetScript("OnEvent", function(self, event, ...)
	if (event == "CHAT_MSG_COMBAT_HONOR_GAIN") then
		HonorTracker:CHAT_MSG_COMBAT_HONOR_GAIN(...)
	else
		HonorTracker:CheckDayChange()
	end
end)

eventFrame:RegisterEvent("CHAT_MSG_COMBAT_HONOR_GAIN")
eventFrame:RegisterEvent("PLAYER_PVP_KILLS_CHANGED")
eventFrame:RegisterEvent("PLAYER_PVP_RANK_CHANGED")
eventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
